
# q-expansion of newform 882.3.c.c, downloaded from the LMFDB on 27 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 882
weight = 3
poly_data = [4, 0, 2, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 0, 0, 1], 2], [[1, 0, 1, 0], 1], [[0, 4, 0, 1], 2]]

hecke_ring_character_values = [[785, [1, 0, 0, 0]], [199, [-1, 0, 0, 0]]]
aps_data = [[0, -1, 0, 0], [0, 0, 0, 0], [0, 0, 0, 2], [0, 0, 0, 0], [0, 12, 0, 0], [0, 0, -1, 0], [0, 0, 0, 2], [0, 0, 17, 0], [0, 6, 0, 0], [0, 24, 0, 0], [0, 0, 7, 0], [-47, 0, 0, 0], [0, 0, 0, 28], [31, 0, 0, 0], [0, 0, 0, -34], [0, 54, 0, 0], [0, 0, 0, 34], [0, 0, -48, 0], [-31, 0, 0, 0], [0, -42, 0, 0], [0, 0, 47, 0], [41, 0, 0, 0], [0, 0, 0, -2], [0, 0, 0, 24], [0, 0, 24, 0], [0, 0, 0, 72], [0, 0, -17, 0], [0, 102, 0, 0], [169, 0, 0, 0], [0, 42, 0, 0], [209, 0, 0, 0], [0, 0, 0, -24], [0, -108, 0, 0], [0, 0, -113, 0], [0, -36, 0, 0], [10, 0, 0, 0], [0, 0, -24, 0], [86, 0, 0, 0], [0, 0, 0, 74], [0, 0, 0, -18], [0, 6, 0, 0], [0, 0, 25, 0], [0, -54, 0, 0], [-287, 0, 0, 0], [0, -90, 0, 0], [0, 0, -120, 0], [82, 0, 0, 0], [0, 0, -24, 0], [0, 0, 0, -156], [0, 0, -47, 0], [0, -162, 0, 0], [0, 48, 0, 0], [0, 0, 264, 0], [0, 0, 0, -142], [0, 0, 0, 66], [0, -180, 0, 0], [0, 0, 0, -8], [0, 0, 24, 0], [-337, 0, 0, 0], [0, -174, 0, 0], [0, 0, 113, 0], [0, 0, 0, 40], [0, 0, -41, 0], [0, 0, 0, -88], [0, 0, 169, 0], [0, 168, 0, 0], [-185, 0, 0, 0], [-359, 0, 0, 0], [0, -330, 0, 0], [0, 0, 336, 0], [0, 0, 0, -118], [0, -240, 0, 0], [0, 0, -89, 0], [-289, 0, 0, 0], [7, 0, 0, 0], [0, 0, 0, 202], [0, 162, 0, 0], [0, 0, 47, 0], [0, 66, 0, 0], [0, 0, 241, 0], [0, 0, 0, 8], [407, 0, 0, 0], [0, 114, 0, 0], [0, 0, -97, 0], [0, 0, -312, 0], [0, -90, 0, 0], [0, -78, 0, 0], [25, 0, 0, 0], [0, 0, 0, -32], [521, 0, 0, 0], [0, 0, 0, -90], [0, 0, 0, 358], [127, 0, 0, 0], [0, 444, 0, 0], [-233, 0, 0, 0], [0, 0, 0, 220], [0, 0, 0, 130], [0, 0, 0, 232], [0, 0, 305, 0], [335, 0, 0, 0], [-658, 0, 0, 0], [0, 192, 0, 0], [0, 0, 0, 6], [0, 600, 0, 0], [449, 0, 0, 0], [0, 0, 169, 0], [0, 0, 0, -216], [0, 0, 0, -428], [0, 456, 0, 0], [0, 0, 265, 0], [0, 0, 607, 0], [290, 0, 0, 0], [0, 516, 0, 0], [0, 0, 473, 0], [-58, 0, 0, 0], [0, 678, 0, 0], [0, 0, -439, 0], [0, 0, 0, -144], [0, -312, 0, 0], [0, 114, 0, 0], [0, 0, 481, 0], [-263, 0, 0, 0], [0, 0, 0, -204], [0, 678, 0, 0], [0, 0, 713, 0], [0, -690, 0, 0], [-1106, 0, 0, 0], [0, 0, 0, 280], [0, 0, -247, 0], [0, 0, 23, 0], [-487, 0, 0, 0], [0, 360, 0, 0], [-545, 0, 0, 0], [-770, 0, 0, 0], [0, 0, 0, 70], [0, 0, 407, 0], [0, 0, 0, -376], [0, 0, -264, 0], [0, 0, 0, -6], [0, -666, 0, 0], [0, 0, -288, 0], [0, -426, 0, 0], [38, 0, 0, 0], [0, 486, 0, 0], [0, 0, 481, 0], [0, 0, 0, -100], [0, 0, -719, 0], [0, 0, 0, 500], [0, 0, 144, 0], [0, 468, 0, 0], [-574, 0, 0, 0], [0, 0, 0, -66], [1735, 0, 0, 0], [0, 0, 0, 80], [751, 0, 0, 0], [0, -882, 0, 0], [1015, 0, 0, 0], [0, 0, 0, 446], [0, 0, 1009, 0], [0, 0, 0, -506], [0, -852, 0, 0], [0, -726, 0, 0], [-895, 0, 0, 0], [0, 0, 0, -86], [0, 888, 0, 0], [0, 0, 0, -482], [655, 0, 0, 0], [0, 0, -265, 0]]
